#include <Wire.h> 
#include <Adafruit_LIS3DH.h>
#include <Adafruit_Sensor.h>
#include "Joystick.h" 

/*
 * Here is a list of possible accelerometers and their addresses, taken from learn.adafruit.com
 * 
 * 0x18 LIS3DH
 * 0x19 LIS3DH
 * 0x19 LSM303
 * 0x1C MMA845x
 * 0x1C FXOS8700
 * 0x1C MMA7455L
 * 0x1D ADXL345
 * 0x1D MMA845x
 * 0x1D FXOS8700
 * 0x1D LSM9DS0
 * 0x1D MMA7455L
 * 0x1E FXOS8700
 * 0x1E LSM9DS0
 * 0x1F FXOS8700
 * 0x26 MSA301
 * 0x28 BNO055
 * 0x29 BNO055
 * 0x53 ADXL345
 * 0x68 ICM-20649
 * 0x68 MPU-9250
 * 0x68 MPU-60X0
 * 0x69 ICM-20649
 * 0x69 MPU-9250
 * 0x69 MPU-60X0
 * 0x6A ICM330DHC
 * 0x6A LSM6DS33
 * 0x6A LSM6DSOX
 * 0x6B ICM330DHC
 * 0x6B LSM6DS33
 * 0x6B LSM6DSOX
 * 
 * The accelerometers supported by this program are LIS3DH, MPU6050, ADXL345, ADXL343 (equivalent to ADXL345)
 */

typedef enum {
  LIS3DH, MPU6050, ADXL345
} accel_t;

accel_t variety; // The variety of accelerometer.
int address; // The I2C address of the accelerometer.
Joystick_ joystick; // To enable the Arduino to behave as a joystick.

Adafruit_LIS3DH lis = Adafruit_LIS3DH(); // This is only used if the accelerometer is a LIS3DH.

// Try every I2C address to see which one responds.
// Addresses 0..7 and 120..127 are reserved.
// We are assuming that only an accelerometer is connected.
int discover_accelerometer() {
  int address;
  for (address = 8; address < 120; address++) {
    Wire.beginTransmission(address);
    if (Wire.endTransmission() == 0) {
      return address;
    }
  }
  return 0;
}

// Each variety of accelerometer has its own setup function.

void setup_LIS3DH(int address) {
  lis.begin(address);
  lis.setRange(LIS3DH_RANGE_2_G);
  lis.setDataRate(LIS3DH_DATARATE_50_HZ);
  joystick.setXAxisRange(-16384, 16383);
  joystick.setYAxisRange(-16384, 16383);
  joystick.setZAxisRange(-16384, 16383);
  joystick.begin(false); //false means that data is not transmitted automatically.
}

void setup_MPU6050(int address) {
  Wire.beginTransmission(address);   
  Wire.write(0x6B);                  // Talk to the register 6B
  Wire.write(0x00);                  // Make reset - place a 0 into the 6B register
  Wire.endTransmission();        //end the transmission 
  joystick.setXAxisRange(-16384, 16383);
  joystick.setYAxisRange(-16384, 16383);
  joystick.setZAxisRange(-16384, 16383);
  joystick.begin(false); //false means that data is not transmitted automatically.
}

void setup_ADXL345(int address) {
  // Set ADXL345 in measuring mode
  // Start communicating with the device 
  Wire.beginTransmission(address); 
  // Access / talk to POWER_CTL Register - 0x2D
  Wire.write(0x2D); 
  // Enable measurement
  // (8dec -> 0000 1000 binary) Bit D3 High for measuring enable 
  Wire.write(8); 
  Wire.endTransmission();
  Wire.beginTransmission(address); 
  Wire.write(0x31); // Set the measurement range
  Wire.write(0); // Set 10-bit mode with range -2g..+2g. We should now find that values are in the range -512..+511
  Wire.endTransmission();
  joystick.setXAxisRange(-512, 511);
  joystick.setYAxisRange(-512, 511);
  joystick.setZAxisRange(-512, 511);
  joystick.begin(false); //false means that data is not transmitted automatically.
}

// Each variety of accelerometer has its own measure function.

void measure_LIS3DH(int address) {
  // === Read accelerometer data === //
  lis.read();
  // Store the axis values in the Joystick.
  // This takes care of mapping the values into the desired range.
  joystick.setXAxis(0);
  joystick.setYAxis(0);
  joystick.setZAxis(lis.x);
  // Send the joystick data to the PC.
  joystick.sendState();
}

void measure_MPU6050(int address) {
  // === Read accelerometer data === //
  Wire.beginTransmission(address);
  // Start with register 0x3B (ACCEL_XOUT_H)
  Wire.write(0x3B); 
  Wire.endTransmission();
  // Read 6 registers total, each axis value is stored in 2 registers
  Wire.requestFrom(address, 6, true); 
  float x_axis = ( Wire.read() << 8 | Wire.read());
  float y_axis = ( Wire.read() << 8 | Wire.read());
  float z_axis = ( Wire.read() << 8 | Wire.read());
  // Store the axis values in the Joystick.
  // This takes care of mapping the values into the desired range.
  joystick.setXAxis(0);
  joystick.setYAxis(0);
  joystick.setZAxis(y_axis);
  // Send the joystick data to the PC.
  joystick.sendState();
}

void measure_ADXL345(int address) {
  // === Read accelerometer data === //
  Wire.beginTransmission(address);
  // Start with register 0x32 (ACCEL_XOUT_H)
  Wire.write(0x32); 
  Wire.endTransmission();
  // Read 6 registers total, each axis value is stored in 2 registers
  Wire.requestFrom(address, 6, true); 
  float x_axis = ( Wire.read()| Wire.read() << 8);
  float y_axis = ( Wire.read()| Wire.read() << 8);
  float z_axis = ( Wire.read()| Wire.read() << 8);
  // Store the axis values in the Joystick.
  // This takes care of mapping the values into the desired range.
  joystick.setXAxis(0);
  joystick.setYAxis(0);
  joystick.setZAxis(y_axis);
  // Send the joystick data to the PC.
  joystick.sendState();
}

// The setup function finds out which variety of accelerometer is connected, and at which address.

void setup() {
  Wire.begin();
  address = discover_accelerometer();
  switch (address) {
    case 0x18:
    case 0x19:
      variety = LIS3DH;
      setup_LIS3DH(address);
      break;
    case 0x1D:
    case 0x53:
      variety = ADXL345;
      setup_ADXL345(address);
      break;
    case 0x68:
    case 0x69:
      variety = MPU6050;
      setup_MPU6050(address);
      break;
  }
  delay(10);
}

// The loop function measures and sends accelerometer data to the PC, appropriately for each variety.

void loop() {
  switch (variety) {
    case LIS3DH:
      measure_LIS3DH(address);
      break;
    case MPU6050:
      measure_MPU6050(address);
      break;
    case ADXL345:
      measure_ADXL345(address);
      break;
  }
  delay(10);
}
